# Changelog

All notable changes to this project will be documented in this file.

The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),
and this project mostly adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).

## [2.2.0] - 2020-02-03

### Added
- Added a `default` transition property, which generates a simple `transition` class and includes all the properties from `colors` as well as `opacity`, `box-shadow`, and `transform` (taken from Tailwind 1.2)
- Added a `shadow` transition property (taken from Tailwind 1.2)
- Added `50`, `75`, `150`, `200`, `300`, and `400` transition durations

### Changed
- The `transition-colors` utility now includes the `fill` and `stroke` properties, in addition to `background-color`, `border-color`, and `color` (taken from Tailwind 1.2)

## [2.1.0] - 2019-09-04

### Changed since 2.1.0-beta.2
- Allowed for durations and delays to be defined as numbers but be output as `ms`

### Changed since 2.0.x
- Changed the implementation of the `default` duration, timing function, and delay to prevent generating base styles that change the value of `transition-property`, `transition-duration`, `transition-timing-function`, and `transition-delay` on all elements, in order to prevent conflicts with libraries that don’t expect these properties to differ from their CSS-defined default
- Base styles targeting all elements are still generated if a `default` duration, timing function, and/or delay is set (which is the case by default: the `default` duration is `250ms`), but they define custom properties that are very unlikely to conflict with third-party libraries (e.g. `--transition-duration`); these are then used to set actual properties on elements that have a `transition-[property]` or `transition-[duration]` class
- As a result, it is now possible to only use a transition duration utility (e.g. `transition-500`) on an element to make it transition all its properties; no need for `transition-all` anymore, since the default value of `transition-property` is already `all`

## [2.1.0-beta.2] - 2019-09-01

**Release `2.1.0-beta.1` was deprecated and reverted, so the following changes were made from release `2.0.1`:**

### Changed
- Changed the implementation of the `default` duration, timing function, and delay to prevent generating base styles that change the value of `transition-property`, `transition-duration`, `transition-timing-function`, and `transition-delay` on all elements, in order to prevent conflicts with libraries that don’t expect these properties to differ from their CSS-defined default
- Base styles targeting all elements are still generated if a `default` duration, timing function, and/or delay is set (which is the case by default: the `default` duration is `250ms`), but they define custom properties that are very unlikely to conflict with third-party libraries (e.g. `--transition-duration`); these are then used to set actual properties on elements that have a `transition-[property]` or `transition-[duration]` class
- As a result, it is now possible to only use a transition duration utility (e.g. `transition-500`) on an element to make it transition all its properties; no need for `transition-all` anymore, since the default value of `transition-property` is already `all`

## [2.1.0-beta.1] - 2019-08-01 [DEPRECATED]

**Decided to go in another direction for v2.1, and reverted most of the changes in this release.**

### Added
- Added the ability to set a `default` transition property, which is set to `auto` by default, meaning that it depends on the `default` transition duration: if it’s `0s` or `0ms` then `auto` resolves to `all`, otherwise it resolves to `none`

### Changed
- Changed the `default` transition duration from `250ms` to `0ms`
- As a result of the above, base styles targeting all elements are no longer generated by default since the default property and duration are now the same as the CSS defaults (`all` and `0s`, respectively)
- Therefore, with the new defaults you only need a transition duration utility (e.g. `transition-fast`) on an element to make it transition all its properties (no need for `transition-all`); if you want the previous behaviour, simply set the `default` transition duration to a non-zero value
- Changed the default transition durations from a numeric scale (`250`, `500`, `750`, etc.) to a descriptive scale (`slow`, `medium`, `fast`, etc.)
- Changed the default transition delays to match the new duration values, but with a numeric scale (`0`, `200`, `400`, `600`, `800`, and `1000`)

## [2.0.1] - 2019-05-16

### Changed
- The base styles which set the default duration, timing function, and delay are now applied to pseudo-elements as well (changed the `*` selector to `*, *::before, *::after`)

## [2.0.0] - 2019-05-13

### Changed since 2.0.0-beta.2
- Added support for global variants thanks to Tailwind’s `variants()` helper function

### Added since 1.x
- Tailwind 1.0.0 compatibility

### Changed since 1.x
- The plugin doesn’t accept a config object anymore; instead it finds what it needs in the `theme` and `variants` keys of your config (see `README` for more info)
- Multiple utilities as well as responsive variants are now generated by default (see `README` for more info)
- Shorter class names for transition durations (`transition-duration-500` is now `transition-500`) and timing functions (`transition-timing-ease` is now `transition-ease`)
- Transition utilities now set `transition-property` instead of the `transition` shorthand; as a result, responsive transition utilities don’t reset the duration, timing function or delay anymore
- For the above to work, the plugin now sets some base styles on all elements (changes `transition-property`’s default value of `all` to `none`, and sets the default duration, timing function, and delay defined in the theme)

## [2.0.0-beta.2] - 2019-04-07

### Removed
- Removed `transitionProperty` and `willChange`’s `default` key feature introduced in beta 1; it is confusing because the behaviour is different from the other `default` keys in this plugin, but also because it’s not clear what the classes do... it’s better to be explicit (e.g. `transition-all` and `will-change-contents`)

## [2.0.0-beta.1] - 2019-04-07

### Added
- Tailwind 1.0.0 compatibility
- `transitionProperty` and `willChange` now accept a `default` key that generates a simple `transition` / `will-change` class

### Changed
- The plugin doesn’t accept a config object anymore; instead it finds what it needs in the `theme` and `variants` keys of your config (see `README` for more info)
- Multiple utilities as well as responsive variants are now generated by default (see `README` for more info)
- Shorter class names for transition durations (`transition-duration-500` is now `transition-500`) and timing functions (`transition-timing-ease` is now `transition-ease`)
- Transition utilities now set `transition-property` instead of the `transition` shorthand; as a result, responsive transition utilities don’t reset the duration, timing function or delay anymore
- For the above to work, the plugin now sets some base styles on all elements (changes `transition-property`’s default value of `all` to `none`, and sets the default duration, timing function, and delay defined in the theme)

## [1.0.4] - 2018-11-04

### Added
- Added proper tests with Jest

## [1.0.3] - 2018-11-04

### Added
- Added a changelog and a release script

## [1.0.2] - 2018-11-04

### Fixed
- Fixed escaping of transition delay selectors

## [1.0.1] - 2018-08-14

### Added
- Added `timingFunctions` option
- Added `delays` option

### Fixed
- Fixed escaping in selectors generated by the plugin

## [1.0.0] - 2018-05-06

Initial release

[Unreleased]: https://github.com/benface/tailwindcss-transitions/compare/v2.2.0...HEAD
[2.2.0]: https://github.com/benface/tailwindcss-transitions/compare/v2.1.0...v2.2.0
[2.1.0]: https://github.com/benface/tailwindcss-transitions/compare/v2.1.0-beta.2...v2.1.0
[2.1.0-beta.2]: https://github.com/benface/tailwindcss-transitions/compare/v2.0.1...v2.1.0-beta.2
[2.1.0-beta.1]: https://github.com/benface/tailwindcss-transitions/compare/v2.0.1...v2.1.0-beta.1
[2.0.1]: https://github.com/benface/tailwindcss-transitions/compare/v2.0.0...v2.0.1
[2.0.0]: https://github.com/benface/tailwindcss-transitions/compare/v2.0.0-beta.2...v2.0.0
[2.0.0-beta.2]: https://github.com/benface/tailwindcss-transitions/compare/v2.0.0-beta.1...v2.0.0-beta.2
[2.0.0-beta.1]: https://github.com/benface/tailwindcss-transitions/compare/v1.0.4...v2.0.0-beta.1
[1.0.4]: https://github.com/benface/tailwindcss-transitions/compare/v1.0.3...v1.0.4
[1.0.3]: https://github.com/benface/tailwindcss-transitions/compare/v1.0.2...v1.0.3
[1.0.2]: https://github.com/benface/tailwindcss-transitions/compare/v1.0.1...v1.0.2
[1.0.1]: https://github.com/benface/tailwindcss-transitions/compare/v1.0.0...v1.0.1
[1.0.0]: https://github.com/benface/tailwindcss-transitions/releases/tag/v1.0.0
